define(
      ['datagrid/dgrid/RefreshingGridTracker',
       'dijit/registry', 'dojox/widget/Standby', 'dojo/aspect' ],
      function(RefreshingGridTracker,
            registry, Standby, aspect) {

var cpb = {};

cpb.INFO_MESSAGES_TYPE = "messages";
cpb.ERROR_MESSAGES_TYPE = "errors";
cpb.WARNING_MESSAGES_TYPE = "warnings";


cpb.widgetMap = {};
cpb.sectionIds = new Array();
cpb.activeTabId = {};
cpb.tabsToInitialize = [];
cpb.uiContextId = null;
cpb.screenId = null;
cpb.savedMessages = {};
cpb.clearOtherMsgs = true;
cpb.showLog = false;
//cpb.visitedTabs = [];
//cpb.showHideDivs = [];


cpb.overrideBannerText = function(appTitle, pageTitle)
{
   var banner = registry.byId('bannerWrapper');
   if (banner) {
      var title = appTitle;
      if (pageTitle) {
         title += " &#149; " + pageTitle;
      }
      banner.set('appTitle', title);
   }
};

cpb.appendBannerText = function(pageTitle)
{
   cpb.overrideBannerText(null, pageTitle);
};

/**
 * Registers a widget that is on the current screen.
 * @param widget the widget to register.
 */
cpb.registerWidget = function(widget) {
   cpb.widgetMap[widget.id] = widget;
   if (widget.type == 'SECTION') {
      cpb.addSectionId("section_" + widget.id);
   }
};

cpb.getWidget = function(widgetId) {
   return cpb.widgetMap[widgetId];
};

cpb.addSectionId = function(sectionId) {
   cpb.sectionIds[cpb.sectionIds.length] = sectionId; 
};

/**
 * Executes the specified widgetAction.
 * @param widgetAction the widget action to execute.
 */
cpb.performWidgetAction = function(widgetAction) {
   var widget = cpb.getWidget(widgetAction.widgetId);
//   alert("WidgetAction:: widgetId == " + widgetAction.widgetId + " :: type == " + widgetAction.type + " :: arg == " + widgetAction.arg);
   if (widget) {
      // retrieve the function property by name 
      var func = widget[widgetAction.type];
//     alert(func);
      if (func) {
         // call the function in the scope of the widget with the action as the first argument
         func.call(widget, widgetAction);         
      } else {
         // TODO: remove this later
         alert("Widget '" + widget.id + "' does not support " + widgetAction.type + "()");
      }
   }
};

/**
 * Called to evaluate an expression and return the result.
 * 
 * @param expression  the String expression to evaluate
 * @param callbackFunction Either the function to call with the result, or an object
 *        containing the callback and other DWR call options
 */
cpb.evaluateExpression = function(fieldName, expression, updateContext, callbackFunction) {
   var f = document.forms[0];
   var params = getParamsExcludeTable(f);

   CPBService.evaluateExpression(
      fieldName,
      expression,
      updateContext,
      params, 
      callbackFunction
   );
};

cpb.updateContext = function(fieldName)
{
   var f = cpb.getForm();
   var params = getParamsExcludeTable(f);
   
   CPBService.updateContext (
      fieldName, params, function() { /* don't care. */ }
   );
   cpb.setPageDirty();
};

/**
 * Called when a field value is changed (usually only when that
 * field has some <on-change> actions.
 * @param fieldName the name of the field that changed
 */
cpb.fieldValueChanged = function(fieldName) {
   var f = cpb.getForm();
   var params = getParamsExcludeTable(f);
   
   CPBService.processFieldValueChange(
      fieldName, params, 
      function(actions) {
//         alert("performing change actions because of change to field " + fieldName);
         // While executing this batch of widget actions, 
         // don't clear messages that already exist.
         cpb.setClearOtherMessages(false);
         
         for (var i = 0; i < actions.length; i++) 
         {
            cpb.performWidgetAction(actions[i]);
         }
         
         // go back to the default
         cpb.setClearOtherMessages(true);
            
      }
   );
   
   cpb.setPageDirty();
};

/**
 * Called when a field value is selected (usually only when that
 * field has some <on-select> actions.
 * @param fieldName the name of the field that was selected
 * @param selectedItem the map object of the selected row
 */
cpb.fieldSelected = function(fieldName, selectedItem) {
   var f = cpb.getForm();
   var params = getParamsExcludeTable(f);
   
   // If we're a grid (currently always true), and we're updating any other grids
   // via on-select events, wait for those other grids to refresh completely before
   // letting the user click in us again.
   var field = registry.byId(fieldName);
   if (field && field.showMask) {
      RefreshingGridTracker.addGridWaitingForRefreshes(field);
   }
   
   CPBService.processFieldSelect(
      fieldName, params, selectedItem,
      function(actions) {
         // While executing this batch of widget actions, 
         // don't clear messages that already exist.
         cpb.setClearOtherMessages(false);
         for (var i = 0; i < actions.length; i++) {
            cpb.performWidgetAction(actions[i]);
         }
         RefreshingGridTracker.pruneGridsWaitingForRefreshes();
         // go back to the default
         cpb.setClearOtherMessages(true);
      }
   );
};

cpb.setPageDirty = function() {
   if (window.SaveChangesConfirmation)
   {
      window.SaveChangesConfirmation.setPageDirty(true);
   } 
   else if (window.parent.SaveChangesConfirmation)
   {
      window.parent.SaveChangesConfirmation.setChildPageDirty(true);
   }
};

cpb.elementById = elementById;

cpb.getElement = function(obj) {
   if (cpb.isString(obj)) {
      return cpb.elementById(obj);
   }
   return obj;
};

cpb.isString = function(obj) {
   return (obj.constructor === String);
};

/**
 * Sets the enabled state of the specified element (or id).
 * @param element  the id or element you're looking for
 * @param enabled  true to enable, false to disable
 */
cpb.setEnabled = function(element, enabled) {
   element = cpb.getElement(element);
   if ((element != null) && (typeof(element) != "undefined")) {
     if (element instanceof Array) {
       for (var i = 0; i < element.length; i++) {
           element[i].disabled = !enabled;
       }
     } else {
         element.disabled = !enabled;
     }
   }
};

/**
 * Sets the enabled state of a sequence of elements based on the specified id.
 * @param id   the id you're looking for
 * @param enabled  true to enable, false to disable
 */
cpb.setSequenceEnabled = function(id, enabled) {
   var index = 0;
   var element = elementById(id + "_" + (index++));
   while (element) {
      cpb.setEnabled(element, enabled);
      element = elementById(id + "_" + (index++));
   }
};

function startsWith(str, prefix) {
   return str.indexOf(prefix) === 0;
};

function endsWith(str, suffix) {
 return str.indexOf(suffix, str.length - suffix.length) !== -1;
};

/**
 * Sets the enabled state of the specified element (or id).
 * @param element  the id or element you're looking for
 * @param enabled  true to show, false to hide
 */
cpb.setVisible = function(elementIdOrObj, visible) {
   
   var element = cpb.getElement(elementIdOrObj);
   if (element) {
      element.style.display = visible ? "" : "none";
   }

};

/**
 * Sets the enabled state of the specified element (or id).
 * @param element  the id or element you're looking for
 * @param enabled  true to show, false to hide
 */
cpb.setGridVisible = function(elementIdOrObj, gridId, visible) {
   cpb.consoleLog(gridId + " setGridVisible:: " + visible);
   var element = cpb.getElement(elementIdOrObj);
   var gridName = gridId.replace('.', '__');
   var grid = registry.byId(gridName);
   grid.visible = visible;
   if (element) {
      if ((visible && element.style.display==='none') || (!visible && element.style.display!=='none')) {
         element.style.display = visible ? "" : "none";
         if (visible) { // Grids aren't properly sized if initially hidden
grid.resize();
//            grid.refresh();
         }
      }
   }
};

/**
 * Sets the visible state of a sequence of elements based on the specified id.
 * @param id   the id you're looking for
 * @param enabled  true to show, false to hide
 */
cpb.setSequenceVisible = function(id, visible) {
   var index = 0;
   var element = elementById(id + "_" + (index++));
   while (element) {
      cpb.setVisible(element, visible);
      element = elementById(id + "_" + (index++));
   }
};

/**
 * Refreshes the specified data grid.
 * @param gridName The name of the data grid to refresh
 * @param newValue This field should be null and is ignored.
 */
cpb.refreshDataGrid = function(gridName, form, newValue) {
   cpb.consoleLog("cpb.refreshDataGrid: gridName=='" + gridName + "'");
   registry.byId(gridName).cpb_refresh(true);
};

/**
 * Sets the value of the specified element (or id).
 * @param fieldName the name of the field to set the value of
 * @param newValue  the new value
 */
cpb.setValue = function(fieldName, form, newValue) {
   // If the newValue is a PageModification, then let it handle the modifications.
   
   if (newValue && newValue.className && newValue.className == 'PageModification') {
      PageModification.handle(newValue);
   } else {
      form = cpb.getForm(form);
      
      //Try to get the dijit version first and if undefined, then get regular DOM element
      var element = registry.byId(fieldName);
      if (!element) {
         element = registry.byId(fieldName.replace('.', '_'));
         if (!element) {
            element = form[fieldName];
         }
      }
      
      // This extra check is to account for CDD fields since they convert '.' to '_' in field id's. 
      if ((element == null) || (typeof(element) == "undefined")) {
         element = form[fieldName.replace(".", "_")];
      }
      if ((element != null) && (typeof(element) != "undefined")) {
         if (element.type) {  // a single input field
            var hidden = element.type == 'hidden';
            if (element.baseClass.indexOf("dijitSelect") !== -1) { 
              cpb.selectOption(element, newValue);
              cpb.setPageDirty();
            } else if (element.type == 'text' || hidden || element.type == "textarea")  {
//               element.value = newValue == null ? "" : newValue;
            	element.set("value", newValue);
            	cpb.setPageDirty();
            }  
         } else if (element.length) {  // an array of inputs
            if (element.length > 0) {
               if (element[0].type == 'radio') {
                  cpb.selectRadio(form, fieldName, "" + newValue);   // convert newValue to string
                  cpb.setPageDirty();
               } else if (element[0].type == 'checkbox') {
                  cpb.selectCheckboxes(form, fieldName, eval(newValue));
                  cpb.setPageDirty();
               }
            }
         }
      }
      // This extra check is for read-only fields, since they are rendered as spans and not form widgets.
      else {
         var span = document.getElementById(fieldName);
         if (span && span.nodeName.toUpperCase()==='SPAN') {
            span.innerHTML = newValue!==null ? newValue : '';
         }
      }
   }   
};

/**
 * Replaces the options in a select with new options from
 * the specified list.
 * @param id  the id of the select element
 * @param options  an array of label/value objects
 */
cpb.setDojoSelectOptions = function(id, options) {
   // remember what the previous value was
   var selectedValue = cpb.getOptionValue(id);
   
   // replace the old options with the new options
   var filteringSelectWidget = dijit.byId(id);
   filteringSelectWidget.removeOption(filteringSelectWidget.getOptions());

   filteringSelectWidget.addOption(options);

   // re-select the prior value (if it exists)
   filteringSelectWidget.set('value', selectedValue);
};

/**
 * Replaces a group of radios with new radios based on
 * options from the specified list.
 * @param id    the id of element that contains the radio elements
 * @param name  the common name of the radios
 * @param form  the form (or name of the form)
 * @param options  an array of label/value objects
 * @param inline  true to keep all the radios on the same line
 */
cpb.setRadios = function(id, name, form, options, inline) {
   form = cpb.getForm(form);
   
   if (options && options != null) {
      var parent = elementById(id);
      
      // remember what the previous value was
      var selectedValue = cpb.getSelectedRadioValue(form, name);
      
      var html = "";
      // create new radios
      for (var i = 0; i < options.length; i++) {
         html += "<input type=\"radio\" name=\"" + name + "\""
            + " id=\"" + name + "_" + i + "\""
            + " value=\"" + options[i].value + "\" />"
            + options[i].label 
            + (inline ? "&nbsp;" : "<br/>");
      }
      parent.innerHTML = html;
      
      // re-select the prior value (if it exists)
      if (!cpb.selectRadio(form, name, selectedValue) && options.length > 0) {
         elementById(name + "_0").checked = "true";
      };
   }
};

cpb.setCheckboxes = function(id, name, form, options, inline) {
   form = cpb.getForm(form);
   
   if (options && options != null) {
      var parent = elementById(id);
      
      // remember what the previous value was
      var selectedValuesArray = cpb.getSelectedCheckboxValues(form, name);
      
      var html = "";
      // create new checkboxes
      for (var i = 0; i < options.length; i++) {
         if (options[i].value == null || options[i].value.length == 0)
            continue;
         html += "<input type=\"checkbox\" name=\"" + name + "\""
            + " id=\"" + name + "_" + i + "\""
            + " value=\"" + options[i].value + "\" />"
            + options[i].label 
            + (inline ? "&nbsp;" : "<br/>");
      }
      parent.innerHTML = html;
      
      // re-select the prior value (if it exists)
      for (var i=0; i < selectedValuesArray.length; i++) {
         cpb.selectCheckbox(form, name, selectedValuesArray[i]);
      }
   }
};

/**
 * Expands/collapses all sections.
 */
cpb.setSectionsExpanded = function(expanded) {
   if(typeof(expanded) == 'undefined')
      expanded = true;
   
   setVisible("cpb_collapseAll", expanded);
   setVisible("cpb_expandAll", !expanded);
   
   for (var i = 0; i <= cpb.sectionIds.length; i++) {
      setSectionExpanded(cpb.sectionIds[i], expanded);
   }
};


cpb.hideExpandedSections = function() {
   // Deprecated
   cpb.addClass(document.body, "dialog-modal-open");
};

cpb.restoreHiddenSections = function() {
   // Deprecated
   cpb.removeClass(document.body, "dialog-modal-open");
};

cpb.getClasses = function(element) {
   if (!element)
      return [];
   var className = cpb.trim(element.className || "");
   if (!className)
      return [];
   var classNames = className.split(/\s+/);
   return classNames;
};

cpb.addClass = function(element, newClassName) {
   if (!element || !newClassName) return;
   var classNames = cpb.getClasses(element);
   for (var i=0; i < classNames.length; i++) {
      if (classNames[i] == newClassName)
         return;
   }
   classNames.push(newClassName);
   element.className = classNames.join(" ");
};

cpb.removeClass = function(element, oldClassName) {
   if (!element || !oldClassName) return;
   var classNames = cpb.getClasses(element);
   var newClassNames = [];
   for (var i=0; i < classNames.length; i++) {
      if (classNames[i] && classNames[i] != oldClassName)
         newClassNames.push(classNames[i]);
   }
   element.className = newClassNames.join(" ");
};

/**
 * Gets a map of CPB parameters and values from the form.
 */
cpb.getCPBParams = function(form) {
   var cpbPrefix = "cpb_";    // prefix for CPB params
   return getFieldMap(form, 
      function(element) {
         return (element.type == 'hidden' && 
            (element.name.indexOf(cpbPrefix) == 0));
      });
};

/**
 * Called when a new tab is clicked.  Remembers the active tab, lazy loads data if necessary, and fixes grid sizes.
 */
cpb.tabSelectCallback = function(tabSectionKey, /*dijit/_Widget*/tab) {
   
   // remember which tab is selected
   var tabId = tab.id;
   elementById(tabSectionKey + "_actTab").value = tabSectionKey + ":" + tabId;   
   cpb.activeTabId[tabSectionKey] = tabId;
   
   // Workaround to force dgrids to calculate a size and display when
   // they were originally rendered to display: "none" divs.
   //if (!arrayContains(cpb.visitedTabs, tabId)) {
      //cpb.visitedTabs.push(tabId);
      cpb.consoleLog('cpb.setVisible: Scanning components in tab: ' + tabId);
      cpb.fixGridSizesInElement(tab.domNode);
   //}
   
   // Lazily initialize the newly-selected tab if necessary
   if (getShouldInitializeTab(tabId)) {
      cpb.consoleLog("Initializing lazy tab '" + tabId + "' and any visible lazy child tabs");
      cpb.initializeTab(tabId);
   }
   
};

/**
 * Called when a section is expanded.  Fixes grid sizes.
 */
cpb.sectionExpandCollapseCallback = function(sectionID, expanded) {
   // Force grids to size properly if in initially collapsed sections
   if (expanded) {
      var section = registry.byId(sectionID);
      cpb.fixGridSizesInElement(section.domNode);
   }
};

/**
 * Returns whether a tab should be initialized.  Removes it from the list
 * of tabs to be initialized if it was there.
 */
getShouldInitializeTab = function(tabId) {
   for (var i = 0; i < cpb.tabsToInitialize.length; i++) {
      if (cpb.tabsToInitialize[i] == tabId) {
         cpb.tabsToInitialize.splice(i, 1);
         return true;
      }
   }
   return false;
};

/**
 * Initializes a tab with a lazy-load initialize block.  This function is called by
 * cpb.selectTab() and thus usually has no need to be called directly.
 */
cpb.initializeTab = function(tabId) {

   var tabContentDiv = document.getElementById(tabId);
   var standby = new Standby({ target: tabContentDiv, /*color: '#000000', */opacity: 0.5 });
   document.body.appendChild(standby.domNode);
   standby.show();
   CPBService.tabSelected(tabId, cpb.uiContextId, cpb.screenId,
         function(actions) {
            //alert("performing change actions because tab was selected: " + tabId);
            // While executing this batch of widget actions, 
            // don't clear messages that already exist.
            cpb.setClearOtherMessages(false);
    
            for (var i = 0; i < actions.length; i++) {
               cpb.performWidgetAction(actions[i]);
            }
            
            // go back to the default
            cpb.setClearOtherMessages(true);
            
            standby.hide();
         }
   );
   
};

/**
 * Returns the value of the selected option in a select.
 * @param select  the select element
 */
cpb.getSelectedOptionValue = function(select) {
   return select.options[select.selectedIndex].value; 
};

cpb.getOptionValue = function(id) {
   var value = elementById(id).value;
   if (value == null || value == "")
   {
      var hidden_name = id + "_hidden";
      var hidden = elementById(hidden_name);
//      alert("hidden for " + id + " == " + hidden);
      if (hidden)
         value = hidden.value;
//      alert("value from hidden " + hidden_name + " for field " + id + " == " + value);
   }
   return value;
};

/**
 * Selects the option in a select that has the specified value.
 * @param select  the select element
 * @param value   the value of the option to select
 */
cpb.selectOption = function(select, value) {
//   select = cpb.getElement(select);
   var options = select.options;
   if (options.length <= 1 && value != null)
   {
//      alert("building hidden for field " + select.name + " with value " + value);
      var hidden_name = select.name + "_hidden";
      var hidden = document.createElement("input");
      hidden.type = "hidden";
      hidden.id = hidden_name;
      hidden.value = value;
      document.body.appendChild(hidden);
//      alert("hidden " + hidden_name + " created " + elementById(hidden_name));
   }
   else
   {
      if (value == null) {
         value = "";
      }
      for (var i=0; i < options.length; i++) {
         if (options[i].value == value) {
            if (!options[i].selected) {
               select.attr('value', value);
               if (select.onchange) select.onchange();
            }
            return;
         }
      }
   }
};

/**
 * Returns the value of the selected radio button or null if none selected.
 * @param form       the form containing the radiobuttons
 * @param radioName  the common name of all the radiobuttons
 */
cpb.getSelectedRadioValue = function(form, radioName) {
   form = cpb.getForm(form);
   
   var rbs = cpb.getFormElementArray(form, radioName);
   for (var i=0; i < rbs.length; i++) {
      if (rbs[i].checked) {
         return (rbs[i].value);
      }
   }
   return (null);
};

/**
 * Returns an array of selected checkbox values or an empty array if none selected.
 * @param form          the form containing the checkboxes
 * @param checkboxName  the common name of all the checkboxes
 */
cpb.getSelectedCheckboxValues = function(form, checkboxName) {
   form = cpb.getForm(form);
   var cbs = cpb.getFormElementArray(form, checkboxName);
   var array = new Array();
   for (var i=0; i < cbs.length; i++) {
      if (cbs[i].checked) {
         array[array.length] = cbs[i].value;
      }
   }
   return array;
};

/**
 * Selects the radio button that has the specified value.
 * @param form       the form containing the radiobuttons
 * @param radioName  the common name of all the radiobuttons
 * @param value      the value of the radio to select
 */
cpb.selectRadio = function(form, radioName, value) {
   var rbs = cpb.getFormElementArray(form, radioName);
   for (var i=0; i < rbs.length; i++) {
      if (rbs[i].value == value) {
         rbs[i].checked = true;
         return true;
      }
   }
   return false;
};

/**
 * Selects the checkbox that has the specified value.
 * @param form          the form containing the checkboxes
 * @param checkboxName  the common name of all the checkboxes
 * @param value         the value of the checkbox to select
 */
cpb.selectCheckbox = function(form, checkboxName, value) {
   var cbs = cpb.getFormElementArray(form, checkboxName);
   for (var i=0; i < cbs.length; i++) {
      if (cbs[i].value == value) {
         cbs[i].checked = true;
         return true;
      }
   }
   return false;
};

/**
 * Selects the checkbox that has the specified values.
 * @param form          the form containing the checkboxes
 * @param checkboxName  the common name of all the checkboxes
 * @param values        the value of the checkbox to select
 */
cpb.selectCheckboxes = function(form, checkboxName, values) {
   var cbs = cpb.getFormElementArray(form, checkboxName);
   for (var i=0; i < cbs.length; i++) {
      var checked = false;
      for (var j=0; j < values.length; j++) {
         if (cbs[i].value == values[j]) {
            checked = true;
            break;
         }
      }
      cbs[i].checked = checked;
   }
   return false;
};

/**
 * Returns a form object given a form name, index, or form object.
 * @param form  the form name, index, or form object
 */
cpb.getForm = function(form) {
   if (form === undefined)
   {
      // first check for the cpb_formName
      var formField = dojo.byId('cpb_formName');
      if (formField === undefined || formField == null || formField.value == null)
      {
         return document.forms[0];
      }
      var formName = formField.value;
      var form = dojo.byId(formName);
      return form;
   }
   else
   {
      var type = typeof(form);
      
      // if the form index or name was passed
      if (type == "number" || type == "string") {
         form = document.forms[form];
      }
      return form;
   }
};

/**
 * Returns an array of form elements given an input name.
 * @param form          the form containing the elements
 * @param commonName    the common name of all elements in the group
 */
cpb.getFormElementArray = function(form, commonName, elType) {
   form = cpb.getForm(form);
   
   var els = form[commonName];
   if (els == null) {
      els = new Array(0);
   } else if (!els.length) {  // only found one, so make into an array
      els = new Array(els);
   }
   
   if (typeof(elType) != "undefined") {
      var newEls = new Array();
      var size = 0;
       
      for (var i=0; i < els.length; i++) {
         if (els[i].type == elType) {
            newEls[size++] = els[i];
         }
      }
      return newEls;
   }
   
   return els;
};

cpb.popupform = function(myform, windowname)
{
    if (! window.focus)
        return true;
    var width = 800;
    var height = 600;
    var top = (screen.availHeight - height) / 2;
    var left = (screen.availWidth - width) / 2;
    
    if (top < 0) {
       top = 0;
    }
    if (left < 0) {
       left = 0;
    }
    
    var newwin;
    try {
      newwin = window.open('', windowname, 'height=' + height + ',width=' + width + ',left=' + left + ',top=' + top + ',scrollbars=yes,resizable=yes');
      if (newwin) {
           newwin.focus();
          myform.target=windowname;
      }
        return newwin;
    } catch (err) {
      if (newwin != null) {
         newwin.close();
      }
      return null;
    }
};

cpb.invokeCustomAction = function(actionId, uiContextId, screenId, output)
{
   var form = cpb.getForm();
   
   //save original action here because the form tag is used for other things.
   var savedAction = form.action;
   var savedTarget = form.target;
   
   form.action = "InvokeCustomAction.do?method=invoke&actionId=" + actionId + 
                     "&cpb_uiContextID=" + uiContextId + "&cpb_screenId=" + screenId;
   var success = true;
   if (output === undefined || output == "window") {
      success = cpb.popupform(form, "sasActionPopup");
   } else {
      form.target = "_self";
   }
   if (success) {
      form.submit();
   }
   
 //restore to original action
   form.action = savedAction;
   form.target = savedTarget;
};

cpb.invokeCustomActionAsynch = function (actionId, uiContextId, screenId, output)
{
   var pleaseWaitSpan = cpb.getElement(actionId + "_pleaseWait_span");
   // Show progress indicator
   if (window.PleaseWait && pleaseWaitSpan)
   {
      PleaseWait.show(pleaseWaitSpan);
   }
   
   // use DWR to call method
   var form = cpb.getForm();
   var result = CPBService.invokeCustomAction(actionId, uiContextId, screenId, getParamsExcludeTable(form), 
         function(resultObject) {
            var actions     = resultObject.widgetActionList;
            var contentType = resultObject.contentType;
            var content     = resultObject.content;
            
            if (actions != null) {
               for (var i = 0; i < actions.length; i++) {
                  cpb.performWidgetAction(actions[i]);
               }
            }
            
            // Hide the progress indicator
            if (window.PleaseWait && pleaseWaitSpan)
            {
               PleaseWait.hide(pleaseWaitSpan);
            }
            
            if (contentType != null) {
               contentType = contentType.toLowerCase();
               if (content != null) {
                  if (contentType == "text/javascript" && content != null) {
                     eval(content);
                  } else if (contentType == "text/html") {
                     alert(contentType + " has not been supported yet");
                  } else if (contentType == "text/plain") {
                     alert(contentType + " has not been supported yet");
                  } else {
                     alert(contentType + " has not been supported yet");
                  }
               }
            }
               
   });
   
};

cpb.getFormParams = function(form)
{
   if (form === undefined)
   {
      form = cpb.getForm();
   }
   return getParamsExcludeTable(form);
};

getParamsExcludeTable = function(form) {
   var map = getFieldMap(form, 
      function(element) {
         return isInterestingFieldElement(element) &&
            !(element.type == 'hidden' && (element.name.indexOf("_TC") >= 0));
      });
   
   // Interesting information about DataGrids.  Currently just selected rows
   var grids = cpb.getGridsInElement(document.body);
   for (var i=0; i<grids.length; i++) {
      var keys = grids[i].cpb_getSelectedRowValues();
      for (var j=0; j<keys.length; j++) {
         setMapValue(map, grids[i].cpb_name, keys[j].toString());
      }
   }
   
   return map;
};

/**
 * This function returns the values encoded in an HTML form as string properties in an object. 
 * Disabled form elements, buttons, and other non-value form elements are skipped. 
 * Multi-select elements are returned as an array of string values.
 *
 * @param {Form|String} form - the form from which to retrieve values
 * @param {function} elementTestFunction - the function to use for deciding if an element 
 *    should be included (leave blank for default implementation)
 */
function getFieldMap(form, elementTestFunction) {
   var f = cpb.getForm(form);
   if (!elementTestFunction) {
      elementTestFunction = isInterestingFieldElement;
   }
   
   var result = {};
   
   var els = form.elements;
   for (var i = 0; i < els.length; i++) {
      var element = form.elements[i];
      if (elementTestFunction(element)) {
         var t = element.type;
         if (t == 'radio' || t == 'checkbox') {
            if (element.checked) {
               setMapValue(result, element.name, element.value);
            }
         } else {
            setMapValue(result, element.name, element.value);
         }
      }
   }
   
   return result;
};

/**
 * This function returns true if the element is interesting enough to get it's value.
 * @param {Object} element - the form element to test
 */
function isInterestingFieldElement(element) {
   var t = element.type;
      
   if (t == 'hidden' ||
       t == 'radio' ||
       t == 'checkbox' ||
       t == 'select-one' ||
       t == 'select-multiple' ||
       t == 'text' ||
       t == 'textarea') 
   {
      return !t.disabled;
   }
    
   return false;
};

/**
 * For the named property in the map, set the value. If a value 
 * already exists and it is a string, convert the value to be an 
 * array of values. 
 * @param {Object} map - the map to add values to
 * @param {String} name - the name of the value
 * @param {String} value - the value
 */
function setMapValue(map, name, value) {
   var val = map[name];
   if (isString(val)){
      map[name] = val + "#,#" + value;
   } else {
      map[name] = value;
   }
};


/**
 * Returns true if the object is a string.
 * @param {Object} obj - the object to test
 */
function isString(obj) {
   return (typeof obj == "string" || obj instanceof String);
};

/**
 * Returns true if the object is an Array.
 * @param {Object} obj - the object to test
 */
function isArray(obj) {
   return (obj && obj instanceof Array || typeof obj == "array");
};

PageModification = {};

/**
 * This method is overloaded with the following signatures:
 *    PageModification.handle(pageModification);
 *    PageModification.handle(elementId, pageModification);
 */
PageModification.handle = function(elementId, pageMod)
{
   var pageModification;
   
   if (pageMod === undefined)
   {
      /*
       * user provided just the page modification object
       * and there is no element ID argument...
       */
      pageModification = elementId;
   }
   else
   {
      /*
       * user provided element ID and page modification arguments...
       */
      pageModification = pageMod;
      if (!pageModification.htmlMap)
      {
         pageModification.htmlMap = {};
         pageModification.htmlMap[elementId] = pageModification.html;
      }
   }

   if (pageModification.htmlMap)
   {
      for (var elementId in pageModification.htmlMap)
      {
         var element = document.getElementById(elementId);
         
         if (!element)
         {
            element = document.createElement("DIV");
         }
         
         if (element.style.display == 'none')
         {
            element.style.display = '';
         }
         
         element.innerHTML = pageModification.htmlMap[elementId];
      }
   }
   
   displayMessages(cpb.ERROR_MESSAGES_TYPE, pageModification.errorMessages, cpb.getClearOtherMessages());
   displayMessages(cpb.WARNING_MESSAGES_TYPE, pageModification.warningMessages, cpb.getClearOtherMessages());
   displayMessages(cpb.INFO_MESSAGES_TYPE, pageModification.infoMessages, cpb.getClearOtherMessages());
   
   // then execute any javascript
   if (pageModification.javaScript)
   {
      eval(pageModification.javaScript);
   }
};

cpb.getSavedMessages = function(type)
{
   return cpb.savedMessages[type];
};

cpb.clearSavedMessages = function(type)
{
   if (type)
   {
      delete cpb.savedMessages[type];
   }
   else
   {
      // delete the types that we know about...
      delete cpb.savedMessages[cpb.ERROR_MESSAGES_TYPE];
      delete cpb.savedMessages[cpb.WARNING_MESSAGES_TYPE];
      delete cpb.savedMessages[cpb.INFO_MESSAGES_TYPE];
   }
};

cpb.saveMessages = function(type, messages) {
   cpb.savedMessages[type] = cpb.mergeMessages(
         cpb.getSavedMessages(type), messages);
};

cpb.mergeMessages = function(existingMessages, newMessages)
{
   if (!existingMessages)
   {
      existingMessages = [];
   }
   
   if (newMessages)
   {
      for (var i = 0; i < newMessages.length; i++) 
      {
         existingMessages.push(newMessages[i]);
      }
   }
   
   return existingMessages;
};

/**
 * Returns a string with the leading and trailing white-space removed.
 * @param {String} str - the string to trim
 */
cpb.trim = function(str) {
   var i = 0, j = str.length;
   
   while (i < j && str.charAt(i) <= ' ')
   {
      i++;
   }
   
   while (j > i && str.charAt(j - 1) <= ' ')
   {
      j--;
   }
   
   return str.substring(i, j);
};

/**
 * Dynamically display messages.
 * @param {String} type - the type of message to display [errors|warnings|messages]
 * @param {Message[]} messages - an array of message objects
 * @param {boolean} clearOtherMsgs - if not false, then other messages of the same type will be cleared
 */
function displayMessages(type, messages, clearOtherMsgs) {
   // info messages only have one area for messages
   
   // add saved messages to given messages array...
   messages = cpb.mergeMessages(messages, cpb.getSavedMessages(type));
   
   var el = (cpb.INFO_MESSAGES_TYPE == type) 
      ? elementById(type + "-content")
      : elementById("dyna-" + type + "-content");

   if(el != null) {
      if (messages && (messages.length > 0)) {
         var content = "";
         for (var i = 0; i < messages.length; i++) {
            content += "<div>" + messages[i].text + "</div>";
         }
         
         if (clearOtherMsgs == false) {
            el.innerHTML += content;
         } else {
            el.innerHTML = content;
         } 
         
         // show the message div
         setVisible(type, true);
         
         // scroll to the top if any errors occurred
         if (cpb.ERROR_MESSAGES_TYPE == type) {
            if (Messages !== undefined) {
               Messages.focusErrors();
            }
            else {
               window.scroll(0,0);
            }
         }
      }
      else if (clearOtherMsgs != false)
      {
         el.innerHTML = "";   // clear the dynamic msgs content
         
         // also hide the whole messages box if the static area is empty
         var staticMsgs = elementById(type + "-content");
         if (cpb.trim(staticMsgs.innerHTML) == "") {
            setVisible(type, false);
         }
      }
   }
};

cpb.displayMessages = displayMessages;

function setExpanded(elementId, toggleImgId, expanded) {
  var element = elementById(elementId);
  var toggleImg = elementById(toggleImgId);
  if (element && toggleImg) {
     var hidden = (element.style.display == "none");
     if (hidden == expanded) {
        setVisible(elementId, expanded);
        setToggleImage(toggleImg, !expanded);
        if (toggleImg.afterMethod) {
           toggleImg.afterMethod(expanded);
        }
     }
  }
};

function setSectionExpanded(sectionId, expanded) {
  setExpanded(sectionId + '_body', sectionId + '_toggle_img', expanded);
};

/**
 * Sets the focus to the first input field on the page.
 */
cpb.setFocus = function() {
    for (var formIdx = 0; formIdx < document.forms.length; formIdx++) {
        var form = document.forms[formIdx];
      for (var i = 0; i < form.elements.length; i++) {
            var element = form.elements[i];
            if (element.disabled == true) continue;
            if (element.type == 'radio' && element.checked &&
                isVisible(element))
            {
                element.focus();
                break;
            } else if (element.focus &&
                element.type != 'hidden' && 
                element.type != 'button' &&
                element.type != 'submit' &&
                element.type != 'reset' &&
                isVisible(element))
            {
                try {
                    element.focus();
                } catch (error) {
                    // eat exception if field isn't visible
                }
                break;
            }
        }
    }
};

/**
 * Returns true if the specified element (and its ancestors) are visible.
 */
function isVisible(element) {
    while (element) {
        if (element.style && element.style.display == "none") return false;
        element = element.parentNode;
    }
    return true;
};

// This should be in CPBDataGrid.
cpb.getGridsInElement = function(element)
{
   var grids = [];
   
   var divs = dojo.query('div[id*="gridwrap_"]', element);
   var count = divs ? divs.length : 0;
   for (var i = 0; i < count; i++) {
      var gridId = divs[i].id.replace('gridwrap_', '');
      var grid = registry.byId(gridId);
      if (grid) {
         grids.push(grid);
      }
   }
   
   return grids;
};

/**
 * Sets the clearOtherMessages flag.
 */
cpb.setClearOtherMessages = function(clearOtherMsgs)
{
   this.clearOtherMsgs = clearOtherMsgs;
};

/**
 * Retrieves the clearOtherMessages flag.
 */
cpb.getClearOtherMessages = function()
{
   return this.clearOtherMsgs != false;
};

cpb.fixGridSizesInElement = function(element)
{
   var grids = cpb.getGridsInElement(element);
   for (var i = 0; i < grids.length; i++) {
      var grid = grids[i];
      if (!grid._started) {
         cpb.consoleLog("cpb.fixGridSizesInElement: Starting up grid: " + grid.cpb_name);
         grid.startup();
      }
      else {
         grid.resize();
      }
   }
};

function toggleCollapsed(element, toggleImg) {

   var hidden = (element.style.display == "none");
   //element.style.display = hidden ? "" : "none";
   setToggleImage(toggleImg, !hidden);
   
   require(['dojo/fx'], function(fx) {
      var animation = null;
      var animProps = { node: element };
      if (hidden) {
         animProps.onEnd = function() {
            cpb.fixGridSizesInElement(element);
         };
         animation = fx.wipeIn(animProps);
      }
      else {
         animation = fx.wipeOut(animProps);
      }
      animation.play();
      
      if (toggleImg.afterMethod) {
          toggleImg.afterMethod(hidden);
      }
   });
};

cpb.setCollapsible = function(elementId, toggleImgId, afterMethod, toggleHrefId) {
   var element = elementById(elementId);
   var toggleImg = elementById(toggleImgId);
   
   toggleImg.targetId = elementId;
  var hidden = (element.style.display == "none");
  setToggleImage(toggleImg, hidden);
  toggleImg.style.cursor = "pointer";
  
  if (afterMethod != null) {
     toggleImg.afterMethod = afterMethod;
  }
  
  if (toggleHrefId) {
     var toggleHref = elementById(toggleHrefId);
     toggleHref.toggleImgId = toggleImgId;
     toggleHref.onclick = function() {
        var toggleImg = elementById(this.toggleImgId);
        toggleCollapsed(elementById(toggleImg.targetId), toggleImg);
        return false;         
     }; 
      toggleHref.onkeydown = function(evt) {
        var keycode;
        if (window.event) keycode = window.event.keyCode;
        else if (evt) keycode = evt.which;
        if (keycode == 37 || keycode == 39) { // left or right arrow
           setExpanded(elementId, toggleImgId, keycode == 39);
        }
     };  
  } else {
     toggleImg.onclick = function() {
        toggleCollapsed(elementById(this.targetId), this);
        return false;
     }; 
  }
  
};

cpb.consoleLog = function(msg, type)
{
   if (cpb.showLog == true)
   {
      if (typeof console !== "undefined")
      {
         type = type || 'info';
         if (type==='error' && console.error)
         {
            console.error(msg);
         }
         else if ((type==='warn' || type==='warning') && console.warn)
         {
            console.warn(msg);
         }
         else if (console.info)
         {
            var title = type.charAt(0).toUpperCase() + type.substring(1);
            console.info(title + ": " + msg);
         }
         else if (console.log)
         {
            var title = type.charAt(0).toUpperCase() + type.substring(1);
            console.log(title + ": " + msg);
         }
         else
         {
            alert("Cannot log this message: " + msg);
         }
      }
   }
};

cpb.setScreenPrintMode = function()
{
   // Dojo registers every widget in a registry. When in print mode, we'll end up having duplicates, so we need to add
   // a unique identifier on each id.
//   aspect.before(registry, 'add', function(widget) 
//   {
//      cpb.consoleLog("before registry.add called. Widget with id, " + widget.id);
//      if (widget.id)
//      {
//         widget.id = widget.id + "__" + new Date().getTime();
//      }
//   });
//   aspect.around(registry, 'add', function(addOrig)
//   {
//      cpb.consoleLog("Around for registry.add being initialized.");
//      return function(widget)
//      {
//         cpb.consoleLog("Around advice for registry.add called. Widget with id, " + widget.id);
//         // do nothing else.
//      };
//   });
};

/**
 * Applications can set this to a function, taking a single string parameter.
 * The string is a URL to redirect to.  Parts of CPB that can navigate to a new page
 * will check if this function is non-null; if so, it is called instead of changing
 * the window location directly.  This gives applications built on CPB to check for
 * unsaved changes, etc.
 */
cpb.redirectHandler = null;

/**
 * Applications should set this flag to true in their cpb.redirectHandler.
 */
cpb.isRedirecting = false;

return cpb;

});

function common_handleWindowOnLoad()
{
   if (window.dwr !== undefined)
   {
      dwr.engine.setErrorHandler(dwrErrorHandler);
   }
};

function setEnabled(elementId, enabled) {
   var element = elementById(elementId);
   if ((element != null) && (typeof(element) != "undefined")) {
       element.disabled = !enabled;
   }
};

function setVisible(elementId, visible) {
   var element = elementById(elementId);
   if ((element != null) && (typeof(element) != "undefined")) {
       element.style.display = visible ? "" : "none";
   }
};

//function setToggleImage(toggleImg, collapsed) {
//  if(collapsed) {
//     toggleImg.src   = "<fmt:message key='application.plus.image'/>";
//     toggleImg.alt   = "<fmt:message key='application.collapsableDiv.plus.tooltip.txt'/>";
//     toggleImg.title = "<fmt:message key='application.collapsableDiv.plus.tooltip.txt'/>";
//  } else {
//     toggleImg.src   = "<fmt:message key='application.minus.image'/>";
//     toggleImg.alt   = "<fmt:message key='application.collapsableDiv.minus.tooltip.txt'/>";
//     toggleImg.title = "<fmt:message key='application.collapsableDiv.minus.tooltip.txt'/>";
//  }
//};

/** 
 * PRIVATE function
 * Browser independent way of retrieving a dom element by id
 * @param id   the id you're looking for
 * @param doc  the document to search (or empty for current doc)
 */
elementById = function(id, doc) {
   if (!doc) {
      doc = document;
   }
   if (doc.getElementById) { 
      return doc.getElementById(id); 
   } else {
      return doc.all[id]; 
   } 
};
