define(['cpb', 'dijit/Dialog' ], function(cpb, Dialog) {
         
   var Modal = {};
   
   Modal.windowCount = 1;
   Modal.windowStack = [];
   
   /**
    * Create a new modal window with content loaded from an iframe.
    */
   Modal.show = function(args)
   {
      cpb.consoleLog("Modal.show called.");
      if (!args)
      {
         var msg = "Invalid arguments to Modal.show(args)";
         cpb.consoleLog("ERROR: " + msg);
         throw new Error(msg);
      }
      
      args.ownerWindow = window;
      
      if (args.url)
      {
         if (args.url.indexOf('http') == -1)
         {
            args.url = Global.BASE_URL + args.url;
         }
         cpb.consoleLog("args.url == " + args.url);
      }
      else
      {
         var msg = "No url provided to Modal.show(args)";
         cpb.consoleLog("ERROR: " + msg);
         throw new Error(msg);
      }
      
      if (args.width)
      {
         cpb.consoleLog("args.width == " + args.width);
      }
      else
      {
         args.width = 850;
         cpb.consoleLog("setting args.width to default :: " + args.width);
      }
      
      if (args.height)
      {
         cpb.consoleLog("args.height == " + args.height);
      }
      else
      {
         args.height = 550;
         cpb.consoleLog("setting args.height to default :: " + args.height);
      }
      
      if (args.title)
      {
         cpb.consoleLog("args.title == " + args.title);
      }
      else
      {
         args.title = "";
         cpb.consoleLog("setting args.title to default :: " + args.title);
      }
      
      var winID = 'win_' + (Modal.windowCount++);
      var iframeID = _createIframeID(winID);
      var modalWin = null;
      
      cpb.consoleLog("Creating Modal with id :: " + winID);
      
      // TODO: This is experimental code for resizable Modal windows. Had some trouble getting it working for now. Revisit later.
   //   require(["dojo/dom-construct", "dojox/layout/FloatingPane"], function(domConstruct, floatingPane)
   //   {
   //      var divID = 'div_' + winID;
   //      var divNode = domConstruct.create("div", {id: divID}, window.top.document.body);
   //      
   //      modalWin = new floatingPane({
   //         id: winID, 
   //         title: args.title,
   //         resizable: true,
   //         dockable: false,
   //         style: "position:absolute;width:" +  + args.width + "px;height:" + args.height + "px;",
   //         //content: "<iframe id='" + iframeID + "' src='" + args.url + "' style=\"width: " + args.width + "px; height: " + args.height + "px\"></iframe>",
   //         content: "<iframe id='" + iframeID + "' src='" + args.url + "' style=\"width: 100%; height: 100%\"></iframe>",
   //         resizeAxis: "xy"
   //      }, divNode/*document.createElement('div')*/);
   //      modalWin.startup();
   //      modalWin.show();
   //   });
      // Create the content first.
      var content = _createIframeContent(iframeID, args.url, args.width, args.height);
      
      modalWin = new Dialog({
         id: winID,
         title: args.title,
         _update: function(){ cpb.consoleLog("Modal window was updated."); }
      });
      
      // The placeAt method docs the modal with the parent window. This prevents modals spawned by a modal to be stuck 
      // inside each other.
      modalWin.placeAt(window.top.document.body);
      
      // Set the content after the placeAt method, since placeAt was triggering the url of the iframe a second time. 
      // Setting it after the placeAt causes it to just be triggered once with the set call on the next line.
      modalWin.set("content", content);
      modalWin.show();
      
      // Check for an update listener
      if (args.updateListener && args.updateListenerObj)
      {
         cpb.consoleLog("Registering the following listener function on obj, " + args.updateListenerObj);
         cpb.consoleLog(args.updateListener);
         dojo.connect(modalWin, "_update", args.updateListenerObj, args.updateListener);
      }
      else
      {
         cpb.consoleLog("No update listener function is registered.");
      }
      
      Modal.windowStack.push(modalWin);
      return modalWin;
   };
   
   /**
    * Loads existing div content into a Modal.
    */
   Modal.load = function(args)
   {
      cpb.consoleLog("Modal.load called.");
      if (!args)
      {
         var msg = "Invalid arguments to Modal.load(args)";
         cpb.consoleLog("ERROR: " + msg);
         throw new Error(msg);
      }
      
      args.ownerWindow = window;
      
      if (args.contentID)
      {
   	   cpb.consoleLog("args.contentID == " + args.contentID);
      }
      else
      {
         var msg = "No contentID provided to Modal.load(args)";
         cpb.consoleLog("ERROR: " + msg);
         throw new Error(msg);
      }
      
      if (args.width)
      {
         cpb.consoleLog("args.width == " + args.width);
      }
      else
      {
         args.width = 850;
         cpb.consoleLog("setting args.width to default :: " + args.width);
      }
      
      if (args.height)
      {
         cpb.consoleLog("args.height == " + args.height);
      }
      else
      {
         args.height = 550;
         cpb.consoleLog("setting args.height to default :: " + args.height);
      }
      
      if (args.title)
      {
         cpb.consoleLog("args.title == " + args.title);
      }
      else
      {
         args.title = "";
         cpb.consoleLog("setting args.title to default :: " + args.title);
      }
      
      var winID = 'win_' + (Modal.windowCount++);
      var modalWin = null;
   	   
      cpb.consoleLog("Creating Modal with id :: " + winID);
      // Create the content first.
      var content = _createIframeContent(iframeID, args.url, args.width, args.height);
      
      modalWin = new Dialog({
         id: winID,
         title: args.title,
         _update: function(){ cpb.consoleLog("Modal window was updated."); }
      });
      
      // The placeAt method docs the modal with the parent window. This prevents modals spawned by a modal to be stuck 
      // inside each other.
      modalWin.placeAt(window.top.document.body);
      
      // Set the content after the placeAt method, since placeAt was triggering the url of the iframe a second time. 
      // Setting it after the placeAt causes it to just be triggered once with the set call on the next line.
      modalWin.set("content", content);
      if (args.show == true)
      {
   	   modalWin.show();
      }
      
      
      // Check for an update listener
      if (args.updateListener && args.updateListenerObj)
      {
         cpb.consoleLog("Registering the following listener function on obj, " + args.updateListenerObj);
         cpb.consoleLog(args.updateListener);
         dojo.connect(modalWin, "_update", args.updateListenerObj, args.updateListener);
      }
      else
      {
         cpb.consoleLog("No update listener function is registered.");
      }
      
      Modal.windowStack.push(modalWin);
      return modalWin;
   	
   };
   
   /**
    * Updates the active modal.
    * 
    * @param {String} url The new URL to display.
    * @param {String} title The new title for the modal.
    */
   Modal.updateActiveModal = function(url, title)
   {
      if (Modal.windowStack.length>0)
      {
         var modal = Modal.windowStack[Modal.windowStack.length-1];
         modal.set('title', title);
         var iframeID = _createIframeID(modal.id);
         var iframe = document.getElementById(iframeID);
         iframe.src = url;
         //modal.set('content', _createIframeContent(id, url, width, height);
      }
      else
      {
         cpb.consoleLog('Error: Modal.updateActiveModal called, but no modal is displayed!');
      }
   };
   
   _createIframeContent = function(id, url, width, height)
   {
      return "<iframe id='" + id + "' src='" + url + "' style=\"width: " + width + "px; height: " + height + "px\"></iframe>"; 
   };
   
   _createIframeID = function(dialogID)
   {
      return 'iframe_' + dialogID;
   };
   
   /**
    * @return <code>true</code> if a modal was displayed (and thus hidden),
    *         <code>false</code> otherwise.
    */
   Modal.hideActiveModal = function(isDirty)
   {
      
      cpb.consoleLog("Modal.hideActiveModal called with isDirty :: " + isDirty);
      if (Modal.windowStack.length>0)
      {
         var modalWin = Modal.windowStack.pop();
         cpb.consoleLog("win id  :: " + modalWin.id);
         if (isDirty)
         {
            modalWin._update();
         }
         modalWin.hide();
      }
      else
      {
         cpb.consoleLog("Error: No Modal to hide!");
         return false;
      }
      
      if (isDirty)
      {
         window.top.SaveChangesConfirmation.setPageDirty();
      }
      return true;
   };

   return Modal;
   
});