define([
    'dojo/has',
    '../CommentData',
    '../Comment/Attachment',
    'dojox/lang/functional',
    'dijit/_WidgetBase',
    'dijit/form/Button',
    'dojo/_base/declare',
    'dojo/_base/lang',
    'dojo/dom-class',
    'dojo/dom-construct',
    'dojo/dom-style',
    'dojo/i18n!./nls/validate',
    'dojo/on',
    'dojox/form/Uploader',
    'dojox/form/uploader/FileList',
    'fraudCommons/util/NLSUtil',
    'messages/Messages',
    'dojox/form/uploader/plugins/Flash'
], function(has, CommentData, Attachment, functional, WidgetBase, Button, declare, lang, domClass, domConstruct, domStyle, validate, on, Uploader, FileList, NLSUtil, Messages) {

    var containsEmptyFile = function(array) {
        var containsEmpty = false;
        functional.forEach(array, function(file, key) {
            if(file.size === 0) {
                containsEmpty = true;
            }
        });
        return containsEmpty;
    };

    return declare([WidgetBase], {

        /**
         * Initialize local variables upon construction
         */
        constructor: function( args ) {
            this.commentId = args.commentId;
            this.d_form = args.d_form;
            this.dataCallback = args.dataCallback;
            this.onComplete = args.onComplete;
            this.editForm = args.editForm;
            this.cancelCallback = args.cancelCallback;
            this.emptyFileCallback = args.emptyFileCallback;
            this.attachmentsToRemove = [];
        },

        /**
         * Destroys all widgets
         */
        destroyRecursive: function() {
            this.list.destroyRecursive();
            this.d_upload.destroyRecursive();
            this.uploader.destroyRecursive();
            this.d_cancel.destroyRecursive();
        },

        /**
         * Recursively destroys all widgets in the uploader
         */
        destroy: function() {
            this.destroyRecursive();
        },

        /**
         * Construct the attachment subform from inherited arguments
         */
        buildRendering: function() {
            this.inherited(arguments);
            var that = this;
            var d_Form = this.d_form;
            var isIE = has("ie");

            /* Submit button */
            this.d_upload = new Button({
                label: NLSUtil.resolve(this.editForm ? 'label_update' : 'label_post', validate),
                disabled: this.editForm ? false : true
            });

            /* Cancel button */
            this.d_cancel = new Button({
                label: NLSUtil.resolve('btn_cancel', validate),
                disabled: this.editForm ? false : true
            });

            /* Remove attachment button */
            this.d_remove = new Button({
                label: NLSUtil.resolve('btn_remove', validate)
            });

            var btnDiv = domConstruct.create("div");
            var listDiv = domConstruct.create("div");
            var oldAttachmentsList = domConstruct.create("ul");
            listDiv.style.display = 'none';

            domClass.add(listDiv, "filelisttoupload");
            domClass.add(btnDiv, "uploaderbuttons");
            domClass.add(oldAttachmentsList, "commentattachments");
            domClass.add(this.d_cancel.domNode, "commentcancelbutton");
            domClass.add(this.d_upload.domNode, "commentsubmitbutton");
            domClass.add(this.d_remove.domNode, "attachmentremovebutton");

            var urlString = 'comments';
            if(this.editForm) {
                urlString += '?action=update';
            }

            /* Uploader widget to prompt the user for a file to upload*/
            var u = new dojox.form.Uploader({
                label: NLSUtil.resolve('label_browse', validate) + NLSUtil.resolve('ellipsis', validate),
//                deferredUploading: 1024,
                multiple: isIE <= 9 ? false : true, //force ie9 to upload files individually
                uploadOnSelect: false,
                url: urlString
            });

            domClass.add(u.domNode, "commentbrowsebutton");
            this.uploader = u;

            /* Displays the list of files queued for upload.*/
            var list = new FileList({
                uploader:u,
                headerIndex: NLSUtil.resolve('filelist_index', validate),
                headerType: NLSUtil.resolve('filelist_type', validate),
                headerFilename: NLSUtil.resolve('filelist_filename', validate),
                headerFilesize: NLSUtil.resolve('filelist_filesize', validate)
            });

            this.list = list;

            domConstruct.place(list.domNode, listDiv);
            domConstruct.place(oldAttachmentsList, listDiv);
            domConstruct.place(u.domNode, btnDiv);
            domConstruct.place(this.d_remove.domNode, btnDiv);
            domConstruct.place(this.d_cancel.domNode, btnDiv);
            domConstruct.place(this.d_upload.domNode, btnDiv);
            domConstruct.place(listDiv, d_Form.domNode, "last");
            domConstruct.place(btnDiv, d_Form.domNode, "last");

            this.uploader.startup();
            this.list.startup();

            if(this.editForm && this.attachments.length > 0) {
                domStyle.set(u.domNode, {
                    display: 'none'
                });
                domStyle.set(this.d_remove.domNode, {
                    display: 'block'
                });
                domStyle.set(oldAttachmentsList, {
                    display: 'block'
                });
                domStyle.set(listDiv, {
                    display: 'block'
                });
                functional.forEach(this.attachments, function(attachment, key) {
                    var li = new Attachment({
                        title: attachment.name,
                        href: 'ViewAttachmentServlet?a_id=' + attachment.id
                    });
                    domConstruct.place(li.domNode, oldAttachmentsList, "last");
                });
            } else {
                domStyle.set(this.d_remove.domNode, {
                    display: 'none'
                });
            }

            on(this.d_upload, 'click', function() {
//                domStyle.set(u.domNode, 'disabled', true);
                domStyle.set(that.d_remove.domNode, 'display', 'none');
//                domStyle.set(that.d_form.domNode, 'disabled', true);
//                domStyle.set(that.d_upload.domNode, 'disabled', true);
                domStyle.set(that.d_cancel.domNode, 'disabled', true);
                
                if(that.editForm && that.attachmentsToRemove.length > 0) {
                    CommentData.deleteAttachments(that.attachmentsToRemove, that.commentId, lang.hitch(that, that.upload, that.dataCallback()));
                } else {
                    that.upload(that.dataCallback());
                }
                domStyle.set(listDiv, "display", "none");
            });


            on(this.d_remove, 'click', function() {
                if(that.editForm) {
                    functional.forEach(that.attachments, function(attachment, key) {
                        that.attachmentsToRemove.push(attachment.id);
                    });
                }
                u.set('disabled', false);
                u.disabled = false;
                u.domNode.style.display = 'inline-block';
                listDiv.style.display = 'none';
                this.domNode.style.display = 'none';
                u.reset();
            });

            on(u, "change", function() {
                if(that.uploader.getFileList().length > 0) {
                    if(containsEmptyFile(that.uploader.getFileList()) && that.emptyFileCallback) {
                        that.emptyFileCallback();
                    } else {
//                        u.set('disabled', true);
                        u.domNode.style.display = 'none';
                        listDiv.style.display = 'block';
                        if (that.attachmentsToRemove.length > 0) {
                            domStyle.set(oldAttachmentsList, "display", "none");
                        }
                        domStyle.set(that.d_remove.domNode, {
                            display: 'inline-block'
                        });
                    }
                } else {
                    u.set('disabled', false);
                    u.domNode.style.display = 'inline-block';
                    listDiv.style.display = 'none';
                    domStyle.set(that.d_remove.domNode, {
                        display: 'none'
                    });
                }
            });

            on(this.d_cancel, 'click', this.editForm ? function() {
                that.destroy();
                that.attachmentsToRemove = [];
                if(that.cancelCallback) {
                    that.cancelCallback();
                }
            } : function() {
                if(that.d_form.reset) {
                    that.d_form.reset();
                } else  {
                    that.d_form.textContainer.reset();
                }
                u.set('disabled', false);
                u.reset();
                u.domNode.style.display = 'inline-block';
                listDiv.style.display = 'none';
                that.d_remove.domNode.style.display = "none";
            });
        },

        /**
         * Starts dependent services
         */
        startup: function() {
            this.inherited(arguments);
        },

        /**
         * Upload files - called when client clicks "post" button
         */
        upload: function(data) {
            var that = this;
            var errorHandlerSignal;
            var completeHandlerSignal;

            var completionFunction = function(response) {
                if(errorHandlerSignal) {
                    errorHandlerSignal.remove();
                }

                if(completeHandlerSignal) {
                    completeHandlerSignal.remove();
                }

                if(that.onComplete) {
                    that.onComplete(response);
                }
            };

            /*
             * TODO this is a hack ... apparently to indicate success the server needs to return
             * a textarea containing JSON data you want to return... 
             * we decided the lesser evil is to just assume any error means "we're done" 
             */
            errorHandlerSignal = on(this.uploader, 'error', completionFunction);
            completeHandlerSignal = on(this.uploader, 'complete', completionFunction);

            if(this.uploader.getFileList().length > 0) {
                var that = this;
                try {
                   if(has("ie") === 9 && data) {
                      for(var nm in data) {
                         var _node = domConstruct.create('input', {name:nm, type:"hidden"});
                         _node.value = data[nm];
                         that.uploader._inputs.push(_node);
                      }
                   }
                   that.uploader.upload(data);
                } catch (exception) {
                   Messages.showError(NLSUtil.resolve('error_uploading', validate));
                   that.uploader.reset();
                }
            } else if(this.editForm) {
                CommentData.update(data, that.onComplete);
            } else {
                CommentData.save(data, that.onComplete);
            }
        },

        disableButtons: function(disabled) {
            this.d_upload.set('disabled', disabled);
            this.d_cancel.set('disabled', disabled);
        },

        disableCancelButton: function(disabled) {
            this.d_cancel.set('disabled', disabled);
        },

        disableUploadButton: function(disabled) {
            this.d_upload.set('disabled', disabled);
        }
    });
});
