define([
    'dojo/keys',
    'dojo/dom-class',
    '../LoadingSpinner/LoadingSpinner',
    './AttachmentUploader/AttachmentUploader',
    './Comment/Comment',
    './Comment/Attachment',
    './Comment/ReplyForm',
    './CommentData',
    'dijit/_TemplatedMixin',
    'dijit/_WidgetBase',
    'dijit/_WidgetsInTemplateMixin',
    'dijit/layout/ContentPane',
    'dijit/TitlePane',
    'dojo/_base/declare',
    'dojo/_base/lang',
    'dojo/_base/event',
    'dojo/date/locale',
    'dojo/dom-construct',
    'dojo/dom-style',
    'dojo/i18n!./nls/validate',
    'dojo/on',
    'dojo/query',
    'dojo/text!./templates/CommentManager.html',
    'dojo/window',
    'dojox/lang/functional',
    'fraudCommons/util/NLSUtil'
], function(keys, domClass, LoadingSpinner, AttachmentUploader, Comment, Attachment, ReplyForm, CommentData, TemplatedMixin, WidgetBase, WidgetsInTemplateMixin, ContentPane,
            TitlePane, declare, lang, event, locale,
            domConstruct, domStyle, validate, on, query,
            template, window, functional, NLSUtil ) {

    var replyIdPrefix = "newReply_";
    var alreadyRenderedFlag = false;
    var focusForm;

    var createTitlePane = function(comment, first) {
        var tp = new TitlePane({
            title: comment.subject,
            open: first
        });
        domClass.add(tp.domNode, "topic");
        return tp;
    };

    var createComment = function(data, readOnly, onComplete) {
        var that = this;
        var c = new Comment({
            objectId: data.objectId,
            objectType: data.objectType,
            objectName: data.objectName,
            commentId: data.id,
            parentId: data.parentId,
            subject: data.subject,
            author: data.author,
            editable: data.editable,
            deletable: data.deletable,
            attachmentsEditable: data.editable,
            attachments: data.attachments,
            text: data.text,
            /*date: locale.format(new Date(Date.parse(data.createdOn)), {
                selector: 'date',
                datePattern: that.dateFormat
            }),*/
            date: data.createdOn,
            onComplete: onComplete,
            readOnly: readOnly,
            truncationLength: 200
        });
        return c;
    };

    var createAttachmentUploader = function(form, data, commentcontainer, isNewThread, emptyFileCallback) {
        if(isNewThread) {
            commentcontainer.newCommentObjectId.set('value', data.objectId);
            commentcontainer.newCommentObjectType.set('value', data.objectType);
            commentcontainer.newCommentObjectName.set('value', data.objectName);
        }

        return new AttachmentUploader({
            d_form: form,
            url: 'comments',
            editForm: false,
            commentId: data.id,
            dataCallback:function () {
                return {
                    'objectId': data.objectId,
                    'objectType': data.objectType,
                    'objectName': data.objectName,
                    'parentId': (isNewThread) ? 0 : data.id,
                    'subject': (isNewThread) ? commentcontainer.newCommentSubject.get('value') : data.subject,
                    'text' : (isNewThread) ? commentcontainer.newCommentText.get('value') : form.textContainer.get('value')
                };
            },
            onComplete: function() {
                commentcontainer.loadComments();
            },
            emptyFileCallback: emptyFileCallback
        });
    };

    return declare([WidgetBase, TemplatedMixin, WidgetsInTemplateMixin], {
        templateString: template,

        constructor: function( args ) {
            this.inherited(arguments);
            this.user = args.user;
            this.objectType = args.objectType;
            this.objectId = args.objectId;
            this.objectName = args.objectName;
            this.readOnly = args.readOnly;
            this.emptyFileCallback = args.emptyFileCallback;
            this.sortAscending = args.sortAscending || true;
            this.ownCommentsEditable = args.ownCommentsEditable || true;
            this.allCommentsEditable = args.allCommentsEditable || false;
            this.ownCommentsDeletable = args.ownCommentsDeletable || true;
            this.allCommentsDeletable = args.allCommentsDeletable || false;
            this.commentsAddable = (!args.readOnly && args.commentsAddable) || false;
        },

        buildRendering: function() {
            this.inherited(arguments);
        },

        postMixinProperties: function() {
            this.inherited(arguments);
        },

        startup: function() {
            this.inherited(arguments);
        },

        postCreate: function() {
            this.inherited(arguments);
            if (!this.commentsAddable) {
            	domStyle.set(this.newTopicForm.domNode, {'display': 'none'});
            }
            this.newCommentSubject.set({placeHolder: NLSUtil.resolve('new_comment_subject', validate)});
//            this.newCommentText.set({placeHolder: NLSUtil.resolve('new_comment_text', validate)});
            
            if(this.readOnly) {
               this.set('disabled', true);
            }
            
            this.loadComments();
        },

        loadComments: function() {
            var that = this;
            var completedLoadingFlag = false;

            new LoadingSpinner().target(this.container.getParent()).until(function() { return completedLoadingFlag; }).startSpinners(function() {});

            CommentData.retrieve(that.objectId, that.objectType, that.objectName, function(commentData) {
                completedLoadingFlag = true;

                that._commentData = [
                    {
                        topics: commentData
                    }
                ];
                lang.hitch(that, that.render)();

                that.scrollToMostRecentComment();
            });

        },

        scrollToMostRecentComment: function() {
            if( focusForm ) {
                window.scrollIntoView(focusForm.domNode);
            }
        },

        createForm: function(replies, data, d_target, contentPane, isNewThread, isFirstTopic) {
            var that = this;
            if(isNewThread) {

            } else if(replies && replies.length > 0) {
                functional.forEach(replies, function(reply, key)  {
                    var comment = createComment(
                        reply,
                        that.readOnly,
                        lang.hitch(that, that.loadComments)
                    );
                    if(that.sortAscending) {
                        contentPane.addChild(comment, 0);
                    } else {
                        contentPane.addChild(comment);
                    }
                });
            }

            var d_Form = new ReplyForm({
                objectId: this.objectId,
                objectType: this.objectType,
                objectName: this.objectName,
                subject: data.subject
            });

            if( isFirstTopic === true ) {
                focusForm = d_Form;
            }

            d_Form.textContainer.set("id", replyIdPrefix + data.id);
            var d_uploader = createAttachmentUploader(d_Form, data, this, isNewThread, that.emptyFileCallback);

            d_uploader.startup();
            d_Form.contents = d_uploader;
            if(that.sortAscending) {
                contentPane.addChild(d_Form, 0);
            } else {
                contentPane.addChild(d_Form);
            }

            on(d_Form.textContainer, "change", function() {
                d_uploader.disableButtons(!(d_Form.textContainer.textbox.value.trim().length > 0));
            });

         if (!this.commentsAddable) {
            domStyle.set(d_Form.domNode, {'display': 'none'});
         }
         
            d_target.addChild(contentPane);
        },

        generateReplies: function(replies, topic, d_target, contentPane, isFirstTopic) {
            this.createForm(replies, topic, d_target, contentPane, false, isFirstTopic);
        },

        render: function() {
            var d_target = this.container;
            var newTopic_target = this.newTopicAttachments;
            var that = this;

            this.container.destroyDescendants();
            if(this.d_attachmentUploader) {
                this.d_attachmentUploader.destroyRecursive();
            }

            this.newTopicForm.reset();
            this.d_attachmentUploader = createAttachmentUploader(this.newTopicForm, {
                objectType: this.objectType,
                objectId: this.objectId,
                objectName: this.objectName
            }, this, true, that.emptyFileCallback);
            this.d_attachmentUploader.startup();

            if(!this._commentData || this._commentData[0].topics.length === 0) {
                this.container.domNode.style.display = 'none';
                this.noComments.set('content', NLSUtil.resolve('no_comments', validate));
                this.noComments.domNode.style.display = 'block';
            } else {
                var comments = this._commentData[0].topics;
                this.noComments.domNode.style.display = 'none';
                this.container.domNode.style.display = 'block';

                functional.forEach(comments, function(topic, key) {
                    var isFirstTopic = key == "0";
                    var d_topic = createTitlePane(topic, isFirstTopic);
                    var contentPane = new ContentPane({});
                    var comment = createComment(
                        topic,
                        that.readOnly,
                        lang.hitch(that, that.loadComments));
                    domClass.add(contentPane.domNode, "threadContainer");
                    contentPane.addChild(comment);

                    that.generateReplies(topic.replies, topic, d_topic, contentPane, isFirstTopic);
                    d_target.addChild(d_topic);
                });
            }
            this.container.resize();

            var detectTextForButtonDisable = function() {
                var commentText = that.newCommentText.textbox.value.trim();
                var subjectText = that.newCommentSubject.textbox.value.trim();
                that.d_attachmentUploader.disableCancelButton(!(subjectText.length > 0));
                that.d_attachmentUploader.disableUploadButton(!(subjectText.length > 0 && commentText.length > 0));
            };

            on(this.newCommentText, "change", function() {
                detectTextForButtonDisable();
            });

            on(this.newCommentSubject, "change", function() {
                detectTextForButtonDisable();
                if(that.newCommentSubject.value.length > 0) {
                    that.container.domNode.style.height = '25%';
                    that.container.resize();
                    domStyle.set(that.newCommentText.domNode, {
                        display: 'inline',
                        height: '100%'
                    });
                    that.newCommentText.resize();

//                    window.scrollIntoView(buttons[0]);

                    that.d_attachmentUploader.disableButtons(!(that.newCommentText.textbox.value.length > 0 && that.newCommentSubject.value.length > 0));

                    /**
                     * 2 July, 2013
                     *
                     * In Dojo 1.8, the placeHolder property of TextArea widgets is broken, making its position
                     * on the screen unpredictable. Until either a workaround is found or the bug is fixed, the
                     * placeholder text for new topics has been removed.
                     */
//                    that.newCommentText.set({placeHolder: NLSUtil.resolve('new_comment_text', validate)});
                } else {
                    that.newCommentText.domNode.style.display = 'none';
                    that.container.domNode.style.height = '60%';
                }
                that.container.resize();
                that.newCommentText.resize();
            });

            on(this.newCommentSubject, 'keydown', function(e) {
                if(e.keyCode == keys.ENTER) {
                    event.stop(e);
                }
            });
            
            if(this.readOnly) {
               this.set('disabled', true);
            }
        },
        
        _setDisabledAttr: function(/* Boolean */ b) {
           domStyle.set(this.newTopicForm.domNode, 'display', 'none');
           query(".threadContainer > div.comment:first-child").forEach(function(node) {
              domStyle.set(node, 'display', 'none');
           });
           this._set('disabled', true);
        },
        
        _getDisabledAttr: function() {
           return this.disabled;
        }

    });
});
